#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2017-12-18
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93


from flask import Blueprint, render_template, abort, request
from sqlalchemy import *
from sqlalchemy.orm import sessionmaker
from angel_web.core.json_util import row2json
from angel_web.core.message import *
from traceback import format_exc
from xmlrpc.client import ServerProxy
import json
import re
import os

SessionBuilder = None
job_t = None
jg_t = None
jd_t = None


class JobService(Blueprint):
    """
    Provide job management api for web interface
    """

    def __init__(self, engine, logger, rpc_url, fs_client, fs_script_path, local_script_path):
        global SessionBuilder
        global job_t
        global jg_t
        global jd_t
        super(JobService, self).__init__("job_service", __name__)
        self.engine = engine
        self.logger = logger
        self.rpc_url = rpc_url
        self.fs_client = fs_client
        self.fs_script_path = fs_script_path
        self.local_script_path = local_script_path
        SessionBuilder = sessionmaker(bind=engine)
        metadata = MetaData(engine)
        job_t = Table("job", metadata,
                      Column("id", Integer, primary_key=True),
                      Column('group_id', Integer, ForeignKey('job_group.id')),
                      Column('name', String(64), nullable=False),
                      Column('desc', String(128)),
                      Column('worker_gid', Integer),
                      Column('trigger', String(32)),
                      Column('running_timeout', Integer, nullable=False))
        jg_t = Table("job_group", metadata,
                     Column('id', Integer, primary_key=True),
                     Column('name', String(64), nullable=False),
                     Column('desc', String(128)),
                     Column('first_job_id', Integer, ForeignKey('job.id')))
        jd_t = Table("job_depend", metadata,
                     Column('group_id', Integer, ForeignKey('job_group.id')),
                     Column('job1', Integer, ForeignKey('job.id'), primary_key=True),
                     Column('job2', Integer, ForeignKey('job.id'), primary_key=True))

    def register_router(self):
        """
        Register router of job service
        """

        @self.route("/data/all_jobs", methods=["GET", "POST"])
        def all_jobs():
            try:
                jds = self.engine.execute(jd_t.select()).fetchall()
                job_depends = {}
                for jd in jds:
                    job1 = jd['job1']
                    job2 = jd['job2']
                    if job1 in job_depends:
                        job_depends[job1].append(job2)
                    else:
                        job_depends[job1] = [job2]
                rows = self.engine.execute(job_t.select()).fetchall()
                result = []
                for row in rows:
                    item = dict(row.items())
                    item["depends"] = job_depends.get(row["id"])
                    result.append(item)
                return json.dumps(result)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/update_jobs", methods=["GET", "POST"])
        def update_jobs():
            # obtain changed jobs
            items = json.loads(request.form["json"])
            # vertify items
            for item in items:
                if item.get("id") == '':
                    item["id"] = None
                if item.get("worker_gid") == '':
                    item["worker_gid"] = None
                trigger = item.get("trigger")
                if not item.get("group_id") or not item.get("name") or not trigger or not\
                item.get("running_timeout"):
                    return RESPONSE_EMPTY_FIELD
                if not re.match("^((\d{1,4}|\*)\s){4}(\d{1,4}|\*)$", item.get("trigger")):
                    return FailResponse("Trigger illegal").to_json()
            # save into the db
            try:
                session = SessionBuilder()
                for item in items:
                    if "addFlag" in item:
                        session.execute(job_t.insert(), item)
                    else:
                        where_ = job_t.c.id==item.pop("id")
                        session.execute(job_t.update().where(where_), item)
                session.commit()
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()

        @self.route("/data/delete_jobs", methods=["GET", "POST"])
        def delete_jobs():
            ids = request.form["id"]
            where_ = job_t.c.id.in_(ids.split(","))
            try:
                self.engine.execute(job_t.delete().where(where_))
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/all_jds", methods=["GET", "POST"])
        def all_job_depends():
            try:
                jds = self.engine.execute(jd_t.select()).fetchall()
                return row2json(jds)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/add_jds", methods=["GET", "POST"])
        def add_jds():
            # obtain changed job depends
            items = json.loads(request.form["json"])
            # vertify items
            for item in items:
                if "group_id" not in item or "job1" not in item or "job2" not in item:
                    return RESPONSE_EMPTY_FIELD
            # save into the db
            try:
                session = SessionBuilder()
                for item in items:
                    session.execute(jd_t.insert(), item)
                session.commit()
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()

        @self.route("/data/delete_jds", methods=["GET", "POST"])
        def delete_jds():
            items = json.loads(request.form["json"])
            try:
                session = SessionBuilder()
                for item in items:
                    where_ = and_(jd_t.c.job1 == item["job1"], jd_t.c.job2 == item["job2"])
                    session.execute(jd_t.delete().where(where_))
                session.commit()
                return RESPONSE_SUCCESS
            except Exception as e:
                session.rollback()
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()
            finally:
                session.close()

        @self.route("/data/all_jgs", methods=["GET", "POST"])
        def all_groups():
            try:
                rows = self.engine.execute(jg_t.select()).fetchall()
                return row2json(rows)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/update_jgs", methods=["GET", "POST"])
        def update_jgs():
            # obtain job groups
            items = json.loads(request.form["json"])
            # veritify the input data
            for item in items:
                if item.get("id") == '':
                    item["id"] = None
                if item.get("first_job_id") == '':
                    item["first_job_id"] = None
                if not item.get("name"):
                    return RESPONSE_EMPTY_FIELD
            try:
                session = SessionBuilder()
                for item in items:
                    if "addFlag" in item:
                        session.execute(jg_t.insert(), item)
                    else:
                        where_ = jg_t.c.id==item.pop("id")
                        session.execute(jg_t.update().where(where_), item)
                session.commit()
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()

        @self.route("/data/delete_jgs", methods=["GET", "POST"])
        def delete_jgs():
            ids = request.form["id"]
            try:
                session = SessionBuilder()
                where_ = job_t.c.group_id.in_(ids.split(","))
                session.execute(job_t.delete().where())
                where_ = jg_t.c.id.in_(ids.split(","))
                session.execute(jg_t.delete().where(where_))
                session.commit()
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()

        @self.route("/data/alive_job_groups", methods=["GET", "POST"])
        def alive_job_groups():
            try:
                resp = ServerProxy(self.rpc_url).get_alive_job_groups()
                if resp["code"] == 0:
                    return json.dumps(resp["content"])
                else:
                    return FailResponse(resp["content"])
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/alive_task_groups", methods=["GET", "POST"])
        def alive_task_groups():
            try:
                resp = ServerProxy(self.rpc_url).get_alive_task_groups()
                if resp["code"] == 0:
                    return json.dumps(resp["content"])
                else:
                    return FailResponse(resp["content"])
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/job/apply_group", methods=["GET", "POST"])
        def apply_job_group():
            try:
                gid = int(request.args["gid"])
                resp = ServerProxy(self.rpc_url).apply_job_group(gid)
                if resp["code"] == 0:
                    return RESPONSE_SUCCESS
                else:
                    return FailResponse(resp["content"]).to_json()
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/uploadscript", methods=["GET", "POST"])
        def upload_job_script():
            try:
                job_id = request.form["jid"]
                f = request.files["file"]
                filepath = os.path.join(self.local_script_path, job_id)
                f.save(filepath)
                with open(filepath, "rb") as f:
                    self.fs_client.upload(f, self.fs_script_path, job_id)
                os.remove(filepath)
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2017-12-18
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93


from flask import Blueprint, render_template, abort, request
from sqlalchemy import *
from sqlalchemy.orm import sessionmaker
from angel_web.core.json_util import row2json
from angel_web.core.message import *
from traceback import format_exc
from xmlrpc.client import ServerProxy
import json
import re

SessionBuilder = None
task_t = None
tg_t = None


class TaskService(Blueprint):
    """
    Provide task management api for web interface
    """

    def __init__(self, engine, logger, rpc_url):
        global SessionBuilder
        global task_t
        global tg_t
        super(TaskService, self).__init__("task_service", __name__)
        self.engine = engine
        self.logger = logger
        self.rpc_url = rpc_url
        SessionBuilder = sessionmaker(bind=engine)
        metadata = MetaData(engine)
        task_t = Table("task", metadata,
                      Column("id", Integer, primary_key=True),
                      Column('job_id', Integer, ForeignKey('job.id')),
                      Column('group_id', Integer, ForeignKey('task_group.id')),
                      Column('runtime', Integer, nullable=False),
                      Column('worker_id', Integer),
                      Column('done_time', Integer),
                      Column('state', Integer, nullable=False))
        tg_t = Table("task_group", metadata,
                     Column('id', Integer, primary_key=True),
                     Column('group_id', Integer, ForeignKey('job_group.id')),
                     Column('runtime', Integer, nullable=False),
                     Column('done_time', Integer),
                     Column('state', Integer, nullable=False))

    def register_router(self):
        """
        Register router of task service
        """
        @self.route("/data/all_tasks", methods=["GET", "POST"])
        def all_tasks():
            try:
                rows = self.engine.execute(task_t.select().limit(1000).order_by(task_t.c.id)).fetchall()
                return row2json(rows)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/alive_tasks", methods=["GET", "POST"])
        def alive_tasks():
            try:
                resp = ServerProxy(self.rpc_url).get_alive_tasks()
                if resp["code"] == 0:
                    return json.dumps(resp["content"])
                else:
                    return FailResponse(resp["content"])
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/pending_tasks", methods=["GET", "POST"])
        def pending_tasks():
            try:
                resp = ServerProxy(self.rpc_url).get_pending_tasks()
                if resp["code"] == 0:
                    return json.dumps(resp["content"])
                else:
                    return FailResponse(resp["content"])
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/delete_tasks", methods=["GET", "POST"])
        def delete_tasks():
            ids = request.form["id"]
            where_ = task_t.c.id.in_(ids.split(","))
            try:
                self.engine.execute(task_t.delete().where(where_))
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/all_tgs", methods=["GET", "POST"])
        def all_groups():
            try:
                rows = self.engine.execute(tg_t.select()).fetchall()
                return row2json(rows)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/delete_tgs", methods=["GET", "POST"])
        def delete_tgs():
            ids = request.form["id"]
            try:
                session = SessionBuilder()
                where_ = task_t.c.group_id.in_(ids.split(","))
                session.execute(task_t.delete().where(where_))
                where_ = tg_t.c.id.in_(ids.split(","))
                session.execute(tg_t.delete().where(where_))
                session.commit()
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2018-01-10
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93

import sys
import daemonize

BASE_PATH = "/tmp/angel/web"

HELP = """
This executable script is used to start/stop a angel_web process as a daemon process.
This requires specifying a pid file which is used to interact with the process.
Usage examples:
    {0} start
    {0} stop
    {0} restart
    {0} status
    {0} help
\n
"""

pidfile = os.path.join(BASE_PATH, ".pid")
stdout = os.path.join(BASE_PATH, "web.out")
stderr = os.path.join(BASE_PATH, "web.err")

def serve():
    from angel_web import app
    app.serve_forever()

def main():
    if len(sys.argv) != 2:
        sys.stderr.write("command error, more information:\n" + HELP)
    if not os.path.isdir(BASE_PATH):
        os.makedirs(BASE_PATH)
    action = sys.argv[1]
    if action == "start":
        daemonize.start(serve, pidfile, stdout=stdout, stderr=stderr)
    elif action == "stop":
        daemonize.stop(pidfile)
    elif action == "status":
        daemonize.status(pidfile)
    elif action == "restart":
        daemonize.restart(serve, pidfile, stdout=stdout, stderr=stderr)
    elif action == "help":
        sys.stderr.write(HELP)
    else:
        sys.stderr.write("command error, more information:\n" + HELP)

if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2017-12-27
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93

from flask_socketio import disconnect, emit, join_room, rooms
from flask import Blueprint, request, redirect
from angel_web.core.json_util import row2json
from angel_web.core.message import *
from traceback import format_exc
from threading import Lock
from xmlrpc.client import ServerProxy
from traceback import format_exc
import os

NAMESPACE = "/log"
EVENT = "log"


class LogService(Blueprint):
    """
    Provide log management api for web interface
    Also provide log upload interface for executor
    """

    def __init__(self, logger, rpc_url, fs_client, fs_log_path):
        super(LogService, self).__init__("log_service", __name__)
        self.logger = logger
        self.rpc_url = rpc_url
        self.fs_client = fs_client
        self.fs_log_path = fs_log_path
        self.task_ids = {}
        self.lock = Lock()

    def register_router(self):
        """
        Register router of log service
        """
        @self.route("/log/upload", methods=["GET", "POST"])
        def upload_log():
            task_id = request.form.get("task_id")
            log = request.form.get("log")
            if task_id in self.task_ids:
                emit(EVENT, {"data": log}, namespace=NAMESPACE, room=task_id)
                return RESPONSE_SUCCESS
            else:
                return RESPONSE_FAIL

    def register_socket_router(self, socketio):
        @socketio.on("connect", namespace=NAMESPACE)
        def on_connect():
            task_id = request.args.get("tid")
            if task_id is None:
                disconnect()
            self.logger.debug("--connected: tid=" + task_id)
            # read from filesystem if the log exist in ftp server
            try:
                path = os.path.join(self.fs_log_path, task_id)
                if self.fs_client.isfile(path):
                    self.fs_client.read(self._emit_log(socketio, task_id), self.fs_log_path, task_id)
                    return
            except:
                self.logger.warn(format_exc())
            # rpc master for sending log command
            try:
                resp = ServerProxy(self.rpc_url).add_log(int(task_id))
                if resp["code"] == 0:
                    join_room(task_id)
                    with self.lock:
                        counter = self.task_ids.get(task_id, 0)
                        self.task_ids[task_id] = counter + 1
                    return
                else:
                    message = resp["content"]
            except Exception as e:
                message = str(e)
            msg = "add log monitor fail caused by " + message
            self.logger.warn(msg)
            emit(EVENT, {"data": "<font color='red' size='5'>%s</font>" % msg})

        @socketio.on("disconnect", namespace=NAMESPACE)
        def on_disconnect():
            task_id = request.args.get("tid")
            if task_id is None:
                return RESPONSE_EMPTY_FIELD
            self.logger.debug("--disconnected: tid=" + task_id)
            with self.lock:
                counter = self.task_ids.get(task_id)
                if counter and counter > 1:
                    self.task_ids[task_id] = counter - 1
                elif counter is not None:
                    del self.task_ids[task_id]

        @socketio.on("close_req", namespace=NAMESPACE)
        def on_close():
            disconnect()

    def _emit_log(self, socketio, tid):
        """
        Helper method for sending log to web browser through websocket
        """
        def func(message):
            message = message.decode("utf8")
            emit(EVENT, {"data": message}, namespace=NAMESPACE)
        return func



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2017-12-18
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93


from flask import Blueprint, request
from angel_web.core.message import *
from traceback import format_exc
from sqlalchemy import *
from sqlalchemy.orm import sessionmaker
from angel_web.core.json_util import row2json
from xmlrpc.client import ServerProxy
import json

SessionBuilder = None
worker_t = None
wg_t = None


class ResourceService(Blueprint):
    """
    Provide resource management api for web interface
    """

    def __init__(self, engine, logger, rpc_url):
        global SessionBuilder
        global worker_t
        global wg_t
        super(ResourceService, self).__init__("resource_service", __name__)
        self.engine = engine
        self.logger = logger
        self.rpc_url = rpc_url
        SessionBuilder = sessionmaker(bind=engine)
        metadata = MetaData(engine)
        worker_t = Table("worker", metadata,
                         Column('id', Integer, primary_key = True),
                         Column('group_id', Integer, ForeignKey('worker_group.id')),
                         Column('login_time', Integer),
                         Column('desc', String(128)))
        wg_t = Table("worker_group", metadata,
                     Column("id", Integer, primary_key = True),
                     Column("name", String(64), nullable = False),
                     Column("desc", String(128)))

    def register_router(self):
        """
        Register router of the resource service
        """
        @self.route("/data/all_wgs", methods=["GET", "POST"])
        def all_worker_groups():
            try:
                rows = self.engine.execute(wg_t.select()).fetchall()
                return row2json(rows)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/update_wgs", methods=["GET", "POST"])
        def update_worker_groups():
            # obtain changes worker groups
            items = json.loads(request.form["json"])
            # vertify the input data
            for item in items:
                if item.get("id") == '':
                    item["id"] = None
                if not item.get("name"):
                    return RESPONSE_EMPTY_FIELD
            # save into db
            try:
                session = SessionBuilder()
                for item in items:
                    if "addFlag" in item:
                        session.execute(wg_t.insert(), item)
                    else:
                        where_ = wg_t.c.id==item.pop("id")
                        session.execute(jg_t.update().where(where_), item)
                session.commit()
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()
            return RESPONSE_SUCCESS

        @self.route("/data/delete_wgs", methods=["GET", "POST"])
        def delete_worker_groups():
            ids = request.form["id"]
            where_ = wg_t.c.id.in_(ids.split(","))
            try:
                self.engine.execute(wg_t.delete().where(where_))
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/all_workers", methods=["GET", "POST"])
        def all_workers():
            try:
                rows = self.engine.execute(worker_t.select()).fetchall()
                return row2json(rows)
            except Exception as e:
                self.logger.error(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/update_workers", methods=["GET", "POST"])
        def update_workers():
            # obtain changes workers
            items = json.loads(request.form["json"])
            # vertify the input data
            for item in items:
                if not item.get("id"):
                    return RESPONSE_EMPTY_FIELD
                if item.get("group_id") == '':
                    item["group_id"] = None
            # update worker group
            try:
                session = SessionBuilder()
                for item in items:
                    where_ = worker_t.c.id==item.pop("id")
                    session.execute(worker_t.update().where(where_), item)
                session.commit()
            except Exception as e:
                self.logger.warn(format_exc())
                session.rollback()
                return FailResponse(str(e)).to_json()
            finally:
                session.close()
            return RESPONSE_SUCCESS

        @self.route("/data/delete_workers", methods=["GET", "POST"])
        def delete_workers():
            ids = request.form("id")
            where_ = worker_t.c.id.in_(ids.split(","))
            try:
                self.engine.execute(worker_t.delete().where(where_))
                return RESPONSE_SUCCESS
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/data/alive_workers", methods=["GET", "POST"])
        def alive_workers():
            # obtain worker list from master by rpc
            try:
                resp = ServerProxy(self.rpc_url).all_alive_workers()
                if resp["code"] == 0:
                    return json.dumps(resp["content"])
                else:
                    return FailResponse(resp["content"])
            except Exception as e:
                self.logger.warn(format_exc())
                return FailResponse(str(e)).to_json()

        @self.route("/resource/off_worker", methods=["GET", "POST"])
        def off_workers():
            return RESPONSE_SUCCESS



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2018-01-10
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93


from setuptools import setup
import sys

setup(
    name="angel_web",
    version="1.0.0",
    author="Lafite93",
    author_email="2273337844@qq.com",
    description="The admin web dashboard of a distribute job scheduler named 'angel'",
    license="Apache",
    keywords="angel admin web dashboard",
    url="https://pypi.python.org/pypi/angel_web",
    packages=["angel_web"],
    scripts=["scripts/angel_web.py"],
    install_requires=[
        "flask",
        "sqlalchemy",
        "flask_socketio",
        "configparser",
        "ftplib"
    ],
    zip_safe=False,
    platforms=["Linux", "Unix"],
    classifiers = [
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
    ],
    entry_points={
        "console_scripts":[
            "angel_web = angel_web.app:serve_forever"
        ]
    }
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         2017-12-18
# @Email        2273337844@qq.com
# @Copyright    © 2017 Lafite93

from flask import Flask, request, render_template
from sqlalchemy import create_engine
from angel_web.job import JobService
from angel_web.resource import ResourceService
from angel_web.task import TaskService
from angel_web.log import LogService
from angel_web.core.ftp_client import FTPClient
from angel_web.core import config_util
from flask_socketio import SocketIO
from logging import config
from traceback import format_exc
import os
import logging
ETC_PATH = '/etc/angel/web/'
HTML_FOLDER = "html"
STATIC_FOLDER = "static"

def serve_forever():
    # setup logger
    logger_config_path = os.path.join(ETC_PATH, "logger.conf")
    logging.config.fileConfig(logger_config_path)
    logger = logging.getLogger("debug")
    # loading setting
    config_path = os.path.join(ETC_PATH, "web.conf")
    if not os.path.isfile(config_path):
        self.logger.error("configuration file %s not exist" % config_path)
    try:
        config = config_util.parse(config_path, "web")
        host = config["host"]
        port = config["port"]
        fs_host = config["fs_host"]
        fs_port = config["fs_port"]
        fs_timeout = config["fs_timeout"]
        fs_name = config["fs_name"]
        fs_pwd = config["fs_password"]
        fs_buffer_size = config["fs_buffer_size"]
        fs_script_path = config["fs_script_path"]
        fs_log_path = config["fs_log_path"]
        local_script_path = config["local_script_path"]
        master_rpc_url = config["rpc_url"]
        db_url = config["db_url"]
        max_overflow = int(config["max_overflow"])
    except Exception as e:
        self.logger.error("config file '%s' error: %s" %(config_path, format_exc()))
        raise
    try:
        # create work directory
        if not os.path.isdir(local_script_path):
            os.makedirs(local_script_path)
        # init db engine
        engine = create_engine(db_url, max_overflow=max_overflow)
        # create app and register router
        app = Flask(__name__, template_folder=HTML_FOLDER, static_folder=STATIC_FOLDER, static_url_path="")
        # init filesystem client
        fs_client = FTPClient(fs_host, fs_port, fs_timeout, fs_name, fs_pwd, fs_buffer_size)
        # init job service
        job_service = JobService(engine, logger, master_rpc_url, fs_client, fs_script_path, local_script_path)
        job_service.register_router()
        app.register_blueprint(job_service)
        # init resource service
        resource_service = ResourceService(engine, logger, master_rpc_url)
        resource_service.register_router()
        app.register_blueprint(resource_service)
        # init task service
        task_service = TaskService(engine, logger, master_rpc_url)
        task_service.register_router()
        app.register_blueprint(task_service)
        # init log service
        log_service = LogService(logger, master_rpc_url, fs_client, fs_log_path)
        log_service.register_router()
        app.register_blueprint(log_service)
        # setup router of html pages
        register_router(app)
        # init websocket
        socketio = SocketIO(app)
        log_service.register_socket_router(socketio)
        # run the service
        socketio.run(app, host, port)
    except:
        self.logger.error(format_exc())
        raise


def register_router(app):
    """
    Register router of the web service
    """
    @app.route("/", methods=["GET", "POST"])
    @app.route("/html/", methods=["GET", "POST"])
    def index():
        return render_template("index.html")

    @app.route("/html/<page>", methods=["GET"])
    def send_page(page):
        return render_template(page, **request.args.to_dict())

    @app.route("/static/<path>", methods=["GET", "POST"])
    def send_static_file(path):
        return app.send_static_file(path)

